@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: delaunay.c 40486 2017-10-18 14:26:45Z rouault $
  *
  * Project:  GDAL algorithms
  * Purpose:  Delaunay triangulation
@@ -50,7 +49,7 @@
 #include <ctype.h>
 #include <math.h>
 
-CPL_CVSID("$Id: delaunay.c 40486 2017-10-18 14:26:45Z rouault $");
+CPL_CVSID("$Id: delaunay.c 818830b37601c33fb53567f86949c8edc1157592 2018-10-06 19:02:19 +0200 Even Rouault $")
 
 #if defined(INTERNAL_QHULL) || defined(EXTERNAL_QHULL)
 #define HAVE_INTERNAL_OR_EXTERNAL_QHULL 1
@@ -328,12 +327,25 @@ int  GDALTriangulationComputeBarycentricCoefficients(GDALTriangulation* psDT,
         double dfY3 = padfY[psFacet->anVertexIdx[2]];
         /* See https://en.wikipedia.org/wiki/Barycentric_coordinate_system */
         double dfDenom = (dfY2 - dfY3) * (dfX1 - dfX3) + (dfX3 - dfX2) * (dfY1 - dfY3);
-        psCoeffs->dfMul1X = (dfY2  - dfY3) / dfDenom;
-        psCoeffs->dfMul1Y = (dfX3  - dfX2) / dfDenom;
-        psCoeffs->dfMul2X = (dfY3  - dfY1) / dfDenom;
-        psCoeffs->dfMul2Y = (dfX1  - dfX3) / dfDenom;
-        psCoeffs->dfCstX = dfX3;
-        psCoeffs->dfCstY = dfY3;
+        if( fabs(dfDenom) < 1e-5 )
+        {
+            // Degenerate triangle
+            psCoeffs->dfMul1X = 0.0;
+            psCoeffs->dfMul1Y = 0.0;
+            psCoeffs->dfMul2X = 0.0;
+            psCoeffs->dfMul2Y = 0.0;
+            psCoeffs->dfCstX = 0.0;
+            psCoeffs->dfCstY = 0.0;
+        }
+        else
+        {
+            psCoeffs->dfMul1X = (dfY2  - dfY3) / dfDenom;
+            psCoeffs->dfMul1Y = (dfX3  - dfX2) / dfDenom;
+            psCoeffs->dfMul2X = (dfY3  - dfY1) / dfDenom;
+            psCoeffs->dfMul2Y = (dfX1  - dfX3) / dfDenom;
+            psCoeffs->dfCstX = dfX3;
+            psCoeffs->dfCstY = dfY3;
+        }
     }
     return TRUE;
 }
@@ -429,6 +441,12 @@ int GDALTriangulationFindFacetBruteForce(const GDALTriangulation* psDT,
         double l1, l2, l3;
         const GDALTriBarycentricCoefficients* psCoeffs =
                                     &(psDT->pasFacetCoefficients[nFacetIdx]);
+        if( psCoeffs->dfMul1X == 0.0 && psCoeffs->dfMul2X == 0.0 &&
+            psCoeffs->dfMul1Y == 0.0 && psCoeffs->dfMul2Y == 0.0 )
+        {
+            // Degenerate triangle
+            continue;
+        }
         l1 = BARYC_COORD_L1(psCoeffs, dfX, dfY);
         if( l1 < -EPS )
         {
@@ -524,6 +542,12 @@ int GDALTriangulationFindFacetDirected(const GDALTriangulation* psDT,
         const GDALTriFacet* psFacet = &(psDT->pasFacets[nFacetIdx]);
         const GDALTriBarycentricCoefficients* psCoeffs =
                                 &(psDT->pasFacetCoefficients[nFacetIdx]);
+        if( psCoeffs->dfMul1X == 0.0 && psCoeffs->dfMul2X == 0.0 &&
+            psCoeffs->dfMul1Y == 0.0 && psCoeffs->dfMul2Y == 0.0 )
+        {
+            // Degenerate triangle
+            break;
+        }
         l1 = BARYC_COORD_L1(psCoeffs, dfX, dfY);
         if( l1 < -EPS )
         {
