@@ -29,7 +29,7 @@
 
 #include "oci_wrapper.h"
 
-CPL_CVSID("$Id: oci_wrapper.cpp 082621472436f1c8006dc8c819bb8bac9f5b967d 2018-03-09 19:48:50Z Even Rouault $")
+CPL_CVSID("$Id: oci_wrapper.cpp 79c519fae99c9d29d132402579a7e3723b457ee0 2018-10-19 11:35:28 -0700 Fengting Chen $")
 
 static const OW_CellDepth ahOW_CellDepth[] = {
     {"8BIT_U",          GDT_Byte},
@@ -1123,15 +1123,19 @@ void OWStatement::WriteCLob( OCILobLocator** pphLocator, char* pszData )
 {
     nNextCol++;
 
-    CheckError( OCIDescriptorAlloc(
+    if (CheckError( OCIDescriptorAlloc(
         poConnection->hEnv,
         (void**) pphLocator,
         OCI_DTYPE_LOB,
         (size_t) 0,
         (dvoid **) nullptr),
-        hError );
+        hError ))
+    {
+        CPLDebug("OCI", "Error in WriteCLob");
+        return;
+    }
 
-    CheckError( OCILobCreateTemporary(
+    if (CheckError( OCILobCreateTemporary(
         poConnection->hSvcCtx,
         poConnection->hError,
         (OCILobLocator*) *pphLocator,
@@ -1140,11 +1144,15 @@ void OWStatement::WriteCLob( OCILobLocator** pphLocator, char* pszData )
         (ub1) OCI_TEMP_CLOB,
         false,
         OCI_DURATION_SESSION ),
-        hError );
+        hError ))
+    {
+        CPLDebug("OCI", "Error in WriteCLob creating temporary lob");
+        return;
+    }
 
     ub4 nAmont = (ub4) strlen(pszData);
 
-    CheckError( OCILobWrite(
+    if (CheckError( OCILobWrite(
         poConnection->hSvcCtx,
         hError,
         *pphLocator,
@@ -1157,7 +1165,11 @@ void OWStatement::WriteCLob( OCILobLocator** pphLocator, char* pszData )
         nullptr,
         (ub2) 0,
         (ub1) SQLCS_IMPLICIT ),
-        hError );
+        hError ))
+    {
+        CPLDebug("OCI", "Error in WriteCLob writing the lob");
+        return;
+    }
 }
 
 void OWStatement::Define( OCIArray** pphData )
@@ -1589,6 +1601,44 @@ char* OWStatement::ReadCLob( OCILobLocator* phLocator )
     return pszBuffer;
 }
 
+// Free OCIDescriptor for the LOB, if it is temporary lob, it is freed too.
+void OWStatement::FreeLob(OCILobLocator* phLocator)
+{
+    boolean        is_temporary;
+
+    if (phLocator == nullptr)
+        return;
+
+    if( CheckError( OCILobIsTemporary(
+        poConnection->hEnv,
+        hError,
+        phLocator,
+        &is_temporary), 
+        hError))
+    {
+        CPLDebug("OCI", "OCILobIsTemporary failed");
+        OCIDescriptorFree( phLocator, OCI_DTYPE_LOB );
+        return;
+    }
+
+    if(is_temporary)
+    {
+      if( CheckError( OCILobFreeTemporary(
+        poConnection->hSvcCtx,
+        hError,
+        phLocator),
+        hError))
+      {
+        CPLDebug("OCI", "OCILobFreeTemporary failed");
+        OCIDescriptorFree( phLocator, OCI_DTYPE_LOB );
+        return;
+      }
+
+    }
+
+    OCIDescriptorFree( phLocator, OCI_DTYPE_LOB );
+}
+
 void OWStatement::BindName( const char* pszName, int* pnData )
 {
     OCIBind* hBind = nullptr;
