@@ -33,6 +33,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <algorithm>
+#include <limits>
 #include <memory>
 #include <vector>
 
@@ -49,7 +50,7 @@
 
 /*! @cond Doxygen_Suppress */
 
-CPL_CVSID("$Id: vrtrasterband.cpp 6d2e6ac6582e121797177feb735bb9a3b853918a 2018-03-13 21:03:59Z Even Rouault $")
+CPL_CVSID("$Id: vrtrasterband.cpp 35bffe8063631f82da77398a37900c784b2b414f 2018-11-02 22:54:20 +0100 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -620,10 +621,26 @@ CPLXMLNode *VRTRasterBand::SerializeToXML( const char *pszVRTPath )
     if( m_bNoDataValueSet )
     {
         if (CPLIsNan(m_dfNoDataValue))
+        {
             CPLSetXMLValue( psTree, "NoDataValue", "nan");
+        }
+        else if( eDataType == GDT_Float32 &&
+                 m_dfNoDataValue == -std::numeric_limits<float>::max() )
+        {
+            // To avoid rounding out of the range of float
+            CPLSetXMLValue( psTree, "NoDataValue", "-3.4028234663852886e+38");
+        }
+        else if( eDataType == GDT_Float32 &&
+                 m_dfNoDataValue == std::numeric_limits<float>::max() )
+        {
+            // To avoid rounding out of the range of float
+            CPLSetXMLValue( psTree, "NoDataValue", "3.4028234663852886e+38");
+        }
         else
+        {
             CPLSetXMLValue( psTree, "NoDataValue",
                             CPLSPrintf( "%.16g", m_dfNoDataValue ) );
+        }
     }
 
     if( m_bHideNoDataValue )
@@ -775,6 +792,11 @@ CPLXMLNode *VRTRasterBand::SerializeToXML( const char *pszVRTPath )
 CPLErr VRTRasterBand::SetNoDataValue( double dfNewValue )
 
 {
+    if( eDataType == GDT_Float32 )
+    {
+        dfNewValue = GDALAdjustNoDataCloseToFloatMax(dfNewValue);
+    }
+
     m_bNoDataValueSet = TRUE;
     m_dfNoDataValue = dfNewValue;
 
