@@ -8,6 +8,8 @@
  ******************************************************************************
  * Copyright (c) 2007, Adam Nowacki
  * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2017, Dmitry Baryshnikov, <polimax@mail.ru>
+ * Copyright (c) 2017, NextGIS, <info@nextgis.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -44,15 +46,15 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: gdalwmsdataset.cpp 37949 2017-04-10 20:53:07Z lplesea $");
+CPL_CVSID("$Id: gdalwmsdataset.cpp 4acd10754e950208aeca32134bd83b6df4aa6805 2017-12-20 11:30:20Z Dmitry Baryshnikov $")
 
 /************************************************************************/
 /*                           GDALWMSDataset()                           */
 /************************************************************************/
 GDALWMSDataset::GDALWMSDataset() :
-    m_mini_driver(NULL),
-    m_cache(NULL),
-    m_poColorTable(NULL),
+    m_mini_driver(nullptr),
+    m_cache(nullptr),
+    m_poColorTable(nullptr),
     m_data_type(GDT_Byte),
     m_block_size_x(0),
     m_block_size_y(0),
@@ -61,8 +63,8 @@ GDALWMSDataset::GDALWMSDataset() :
     m_offline_mode(0),
     m_http_max_conn(0),
     m_http_timeout(0),
-    m_http_options(NULL),
-    m_tileOO(NULL),
+    m_http_options(nullptr),
+    m_tileOO(nullptr),
     m_clamp_requests(true),
     m_unsafeSsl(false),
     m_zeroblock_on_serverexceptions(0),
@@ -119,7 +121,7 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
     if (pszReferer[0] != '\0')
         m_osReferer = pszReferer;
 
-    if (ret == CE_None) {
+    {
         const char *pszHttpZeroBlockCodes = CPLGetXMLValue(config, "ZeroBlockHttpCodes", "");
         if (pszHttpZeroBlockCodes[0] == '\0') {
             m_http_zeroblock_codes.insert(204);
@@ -228,17 +230,31 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
         }
     }
 
+    CPLXMLNode *service_node = CPLGetXMLNode(config, "Service");
+    if (service_node == nullptr) {
+        CPLError(CE_Failure, CPLE_AppDefined,
+            "GDALWMS: No Service specified.");
+        return CE_Failure;
+    }
+
     if (ret == CE_None) {
         CPLXMLNode *cache_node = CPLGetXMLNode(config, "Cache");
-        if (cache_node != NULL) {
+        if (cache_node != nullptr) {
             m_cache = new GDALWMSCache();
-            if (m_cache->Initialize(cache_node) != CE_None) {
+            if (m_cache->Initialize(CPLGetXMLValue(service_node, "ServerUrl", nullptr),
+                                    cache_node) != CE_None) {
                 delete m_cache;
-                m_cache = NULL;
+                m_cache = nullptr;
                 CPLError(CE_Failure, CPLE_AppDefined,
                     "GDALWMS: Failed to initialize cache.");
                 ret = CE_Failure;
             }
+            else {
+                // NOTE: Save cache path to metadata. For example, this is
+                // useful for deleting a cache folder when removing dataset or
+                // to fill the cache for specified area and zoom levels
+                SetMetadataItem("CACHE_PATH", m_cache->CachePath(), nullptr);
+            }
         }
     }
 
@@ -255,12 +271,6 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
     }
 
     // Initialize the minidriver, which can set parameters for the dataset using member functions
-    CPLXMLNode *service_node = CPLGetXMLNode(config, "Service");
-    if (service_node == NULL) {
-        CPLError(CE_Failure, CPLE_AppDefined,
-            "GDALWMS: No Service specified.");
-        return CE_Failure;
-    }
 
     const CPLString service_name = CPLGetXMLValue(service_node, "name", "");
     if (service_name.empty()) {
@@ -270,7 +280,7 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
     }
 
     m_mini_driver = NewWMSMiniDriver(service_name);
-    if (m_mini_driver == NULL) {
+    if (m_mini_driver == nullptr) {
         CPLError(CE_Failure, CPLE_AppDefined,
             "GDALWMS: No mini-driver registered for '%s'.", service_name.c_str());
         return CE_Failure;
@@ -282,7 +292,7 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
     {
         CPLError(CE_Failure, CPLE_AppDefined, "GDALWMS: Failed to initialize minidriver.");
         delete m_mini_driver;
-        m_mini_driver = NULL;
+        m_mini_driver = nullptr;
         ret = CE_Failure;
     }
     else
@@ -334,7 +344,7 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
         if (ret == CE_None)
         {
             CPLXMLNode *data_window_node = CPLGetXMLNode(config, "DataWindow");
-            if (data_window_node == NULL && m_bNeedsDataWindow)
+            if (data_window_node == nullptr && m_bNeedsDataWindow)
             {
                 CPLError(CE_Failure, CPLE_AppDefined, "GDALWMS: DataWindow missing.");
                 ret = CE_Failure;
@@ -525,7 +535,8 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
                 double scale = 0.5;
                 for (int j = 0; j < nOverviews; ++j)
                 {
-                    band->AddOverview(scale);
+                    if( !band->AddOverview(scale) )
+                        break;
                     band->m_color_interp = color_interp;
                     scale *= 0.5;
                 }
@@ -550,20 +561,20 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
     if (ret == CE_None) {
         // Data values are attributes, they include NoData Min and Max
         // TODO: document those options
-        if (NULL!=CPLGetXMLNode(config,"DataValues")) {
-            const char *nodata=CPLGetXMLValue(config,"DataValues.NoData",NULL);
-            if (nodata!=NULL) WMSSetNoDataValue(nodata);
-            const char *min=CPLGetXMLValue(config,"DataValues.min",NULL);
-            if (min!=NULL) WMSSetMinValue(min);
-            const char *max=CPLGetXMLValue(config,"DataValues.max",NULL);
-            if (max!=NULL) WMSSetMaxValue(max);
+        if (nullptr!=CPLGetXMLNode(config,"DataValues")) {
+            const char *nodata=CPLGetXMLValue(config,"DataValues.NoData",nullptr);
+            if (nodata!=nullptr) WMSSetNoDataValue(nodata);
+            const char *min=CPLGetXMLValue(config,"DataValues.min",nullptr);
+            if (min!=nullptr) WMSSetMinValue(min);
+            const char *max=CPLGetXMLValue(config,"DataValues.max",nullptr);
+            if (max!=nullptr) WMSSetMaxValue(max);
         }
     }
 
     if (ret == CE_None) {
         if (!m_projection.size()) {
             const char *proj = m_mini_driver->GetProjectionInWKT();
-            if (proj != NULL) {
+            if (proj != nullptr) {
                 m_projection = proj;
             }
         }
@@ -588,7 +599,7 @@ CPLErr GDALWMSDataset::IRasterIO(GDALRWFlag rw, int x0, int y0, int sx, int sy,
     CPLErr ret;
 
     if (rw != GF_Read) return CE_Failure;
-    if (buffer == NULL) return CE_Failure;
+    if (buffer == nullptr) return CE_Failure;
     if ((sx == 0) || (sy == 0) || (bsx == 0) || (bsy == 0) || (band_count == 0)) return CE_None;
 
     m_hint.m_x0 = x0;
@@ -661,10 +672,10 @@ CPLErr GDALWMSDataset::AdviseRead(int x0, int y0,
                                   char **options) {
 //    printf("AdviseRead(%d, %d, %d, %d)\n", x0, y0, sx, sy);
     if (m_offline_mode || !m_use_advise_read) return CE_None;
-    if (m_cache == NULL) return CE_Failure;
+    if (m_cache == nullptr) return CE_Failure;
 
     GDALRasterBand *band = GetRasterBand(1);
-    if (band == NULL) return CE_Failure;
+    if (band == nullptr) return CE_Failure;
     return band->AdviseRead(x0, y0, sx, sy, bsx, bsy, bdt, options);
 }
 
@@ -676,7 +687,7 @@ char **GDALWMSDataset::GetMetadataDomainList()
 {
     return BuildMetadataDomainList(GDALPamDataset::GetMetadataDomainList(),
                                    TRUE,
-                                   "WMS", NULL);
+                                   "WMS", nullptr);
 }
 
 /************************************************************************/
@@ -685,10 +696,10 @@ char **GDALWMSDataset::GetMetadataDomainList()
 const char *GDALWMSDataset::GetMetadataItem( const char * pszName,
                                              const char * pszDomain )
 {
-    if( pszName != NULL && EQUAL(pszName, "XML") &&
-        pszDomain != NULL && EQUAL(pszDomain, "WMS") )
+    if( pszName != nullptr && EQUAL(pszName, "XML") &&
+        pszDomain != nullptr && EQUAL(pszDomain, "WMS") )
     {
-        return (m_osXML.size()) ? m_osXML.c_str() : NULL;
+        return (m_osXML.size()) ? m_osXML.c_str() : nullptr;
     }
 
     return GDALPamDataset::GetMetadataItem(pszName, pszDomain);
@@ -697,10 +708,10 @@ const char *GDALWMSDataset::GetMetadataItem( const char * pszName,
 // Builds a CSL of options or returns the previous one
 const char * const * GDALWMSDataset::GetHTTPRequestOpts()
 {
-    if (m_http_options != NULL)
+    if (m_http_options != nullptr)
         return m_http_options;
 
-    char **opts = NULL;
+    char **opts = nullptr;
     if (m_http_timeout != -1)
         opts = CSLAddString(opts, CPLOPrintf("TIMEOUT=%d", m_http_timeout));
 
@@ -724,4 +735,3 @@ const char * const * GDALWMSDataset::GetHTTPRequestOpts()
     m_http_options = opts;
     return m_http_options;
 }
-
