@@ -35,7 +35,7 @@
 
 #include <cstdlib>
 
-CPL_CVSID("$Id: ogrdxfwriterlayer.cpp 36883 2016-12-15 13:31:12Z rouault $");
+CPL_CVSID("$Id: ogrdxfwriterlayer.cpp 8506be5ef6fc0a87e4446efc45cae79e01c9ee5c 2018-11-26 13:08:08 +0100 Even Rouault $")
 
 /************************************************************************/
 /*                         OGRDXFWriterLayer()                          */
@@ -43,7 +43,7 @@ CPL_CVSID("$Id: ogrdxfwriterlayer.cpp 36883 2016-12-15 13:31:12Z rouault $");
 
 OGRDXFWriterLayer::OGRDXFWriterLayer( OGRDXFWriterDS *poDSIn, VSILFILE *fpIn ) :
     fp(fpIn),
-    poFeatureDefn(NULL),  // TODO(schwehr): Can I move the new here?
+    poFeatureDefn(nullptr),  // TODO(schwehr): Can I move the new here?
     poDS(poDSIn)
 {
     nNextAutoID = 1;
@@ -52,32 +52,8 @@ OGRDXFWriterLayer::OGRDXFWriterLayer( OGRDXFWriterDS *poDSIn, VSILFILE *fpIn ) :
     poFeatureDefn = new OGRFeatureDefn( "entities" );
     poFeatureDefn->Reference();
 
-    OGRFieldDefn  oLayerField( "Layer", OFTString );
-    poFeatureDefn->AddFieldDefn( &oLayerField );
-
-    OGRFieldDefn  oClassField( "SubClasses", OFTString );
-    poFeatureDefn->AddFieldDefn( &oClassField );
-
-    OGRFieldDefn  oExtendedField( "ExtendedEntity", OFTString );
-    poFeatureDefn->AddFieldDefn( &oExtendedField );
-
-    OGRFieldDefn  oLinetypeField( "Linetype", OFTString );
-    poFeatureDefn->AddFieldDefn( &oLinetypeField );
-
-    OGRFieldDefn  oEntityHandleField( "EntityHandle", OFTString );
-    poFeatureDefn->AddFieldDefn( &oEntityHandleField );
-
-    OGRFieldDefn  oTextField( "Text", OFTString );
-    poFeatureDefn->AddFieldDefn( &oTextField );
-
-    OGRFieldDefn  oBlockField( "BlockName", OFTString );
-    poFeatureDefn->AddFieldDefn( &oBlockField );
-
-    OGRFieldDefn  oScaleField( "BlockScale", OFTRealList );
-    poFeatureDefn->AddFieldDefn( &oScaleField );
-
-    OGRFieldDefn  oBlockAngleField( "BlockAngle", OFTReal );
-    poFeatureDefn->AddFieldDefn( &oBlockAngleField );
+    OGRDXFDataSource::AddStandardFields( poFeatureDefn,
+        ODFM_IncludeBlockFields );
 }
 
 /************************************************************************/
@@ -215,7 +191,7 @@ OGRErr OGRDXFWriterLayer::WriteCore( OGRFeature *poFeature )
 /*      "0" - if there is no layer property on the source features.     */
 /* -------------------------------------------------------------------- */
     const char *pszLayer = poFeature->GetFieldAsString( "Layer" );
-    if( pszLayer == NULL || strlen(pszLayer) == 0 )
+    if( pszLayer == nullptr || strlen(pszLayer) == 0 )
     {
         WriteValue( 8, "0" );
     }
@@ -238,7 +214,7 @@ OGRErr OGRDXFWriterLayer::WriteCore( OGRFeature *poFeature )
 
         const char *pszExists =
             poDS->oHeaderDS.LookupLayerProperty( osSanitizedLayer, "Exists" );
-        if( (pszExists == NULL || strlen(pszExists) == 0)
+        if( (pszExists == nullptr || strlen(pszExists) == 0)
             && CSLFindString( poDS->papszLayersToCreate, osSanitizedLayer ) == -1 )
         {
             poDS->papszLayersToCreate =
@@ -265,9 +241,9 @@ OGRErr OGRDXFWriterLayer::WriteINSERT( OGRFeature *poFeature )
     WriteValue( 2, poFeature->GetFieldAsString("BlockName") );
 
     // Write style symbol color
-    OGRStyleTool *poTool = NULL;
+    OGRStyleTool *poTool = nullptr;
     OGRStyleMgr oSM;
-    if( poFeature->GetStyleString() != NULL )
+    if( poFeature->GetStyleString() != nullptr )
     {
         oSM.InitFromFeature( poFeature );
 
@@ -279,25 +255,41 @@ OGRErr OGRDXFWriterLayer::WriteINSERT( OGRFeature *poFeature )
         OGRStyleSymbol *poSymbol = (OGRStyleSymbol *) poTool;
         GBool bDefault;
 
-        if( poSymbol->Color(bDefault) != NULL && !bDefault )
+        if( poSymbol->Color(bDefault) != nullptr && !bDefault )
             WriteValue( 62, ColorStringToDXFColor( poSymbol->Color(bDefault) ) );
     }
     delete poTool;
 
 /* -------------------------------------------------------------------- */
-/*      Write location.                                                 */
+/*      Write location in OCS.                                          */
 /* -------------------------------------------------------------------- */
-    OGRPoint *poPoint = (OGRPoint *) poFeature->GetGeometryRef();
+    int nCoordCount = 0;
+    const double *padfCoords =
+        poFeature->GetFieldAsDoubleList( "BlockOCSCoords", &nCoordCount );
 
-    WriteValue( 10, poPoint->getX() );
-    if( !WriteValue( 20, poPoint->getY() ) )
-        return OGRERR_FAILURE;
-
-    if( poPoint->getGeometryType() == wkbPoint25D )
+    if( nCoordCount == 3 )
     {
-        if( !WriteValue( 30, poPoint->getZ() ) )
+        WriteValue( 10, padfCoords[0] );
+        WriteValue( 20, padfCoords[1] );
+        if( !WriteValue( 30, padfCoords[2] ) )
             return OGRERR_FAILURE;
     }
+    else
+    {
+        // We don't have an OCS; we will just assume that the location of
+        // the geometry (in WCS) is the correct insertion point.
+        OGRPoint *poPoint = poFeature->GetGeometryRef()->toPoint();
+
+        WriteValue( 10, poPoint->getX() );
+        if( !WriteValue( 20, poPoint->getY() ) )
+            return OGRERR_FAILURE;
+
+        if( poPoint->getGeometryType() == wkbPoint25D )
+        {
+            if( !WriteValue( 30, poPoint->getZ() ) )
+                return OGRERR_FAILURE;
+        }
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Write scaling.                                                  */
@@ -323,6 +315,20 @@ OGRErr OGRDXFWriterLayer::WriteINSERT( OGRFeature *poFeature )
         WriteValue( 50, dfAngle ); // degrees
     }
 
+/* -------------------------------------------------------------------- */
+/*      Write OCS normal vector.                                        */
+/* -------------------------------------------------------------------- */
+    int nOCSCount = 0;
+    const double *padfOCS =
+        poFeature->GetFieldAsDoubleList( "BlockOCSNormal", &nOCSCount );
+
+    if( nOCSCount == 3 )
+    {
+        WriteValue( 210, padfOCS[0] );
+        WriteValue( 220, padfOCS[1] );
+        WriteValue( 230, padfOCS[2] );
+    }
+
     return OGRERR_NONE;
 }
 
@@ -339,9 +345,9 @@ OGRErr OGRDXFWriterLayer::WritePOINT( OGRFeature *poFeature )
     WriteValue( 100, "AcDbPoint" );
 
     // Write style pen color
-    OGRStyleTool *poTool = NULL;
+    OGRStyleTool *poTool = nullptr;
     OGRStyleMgr oSM;
-    if( poFeature->GetStyleString() != NULL )
+    if( poFeature->GetStyleString() != nullptr )
     {
         oSM.InitFromFeature( poFeature );
 
@@ -353,12 +359,12 @@ OGRErr OGRDXFWriterLayer::WritePOINT( OGRFeature *poFeature )
         OGRStylePen *poPen = (OGRStylePen *) poTool;
         GBool  bDefault;
 
-        if( poPen->Color(bDefault) != NULL && !bDefault )
+        if( poPen->Color(bDefault) != nullptr && !bDefault )
             WriteValue( 62, ColorStringToDXFColor( poPen->Color(bDefault) ) );
     }
     delete poTool;
 
-    OGRPoint *poPoint = (OGRPoint *) poFeature->GetGeometryRef();
+    OGRPoint *poPoint = poFeature->GetGeometryRef()->toPoint();
 
     WriteValue( 10, poPoint->getX() );
     if( !WriteValue( 20, poPoint->getY() ) )
@@ -392,11 +398,26 @@ CPLString OGRDXFWriterLayer::TextEscape( const char *pszInput )
     for( int i = 0; panInput[i] != 0; i++ )
     {
         if( panInput[i] == '\n' )
+        {
             osResult += "\\P";
+        }
         else if( panInput[i] == ' ' )
+        {
             osResult += "\\~";
+        }
         else if( panInput[i] == '\\' )
+        {
             osResult += "\\\\";
+        }
+        else if( panInput[i] == '^' )
+        {
+            osResult += "^ ";
+        }
+        else if( panInput[i] < ' ' )
+        {
+            osResult += '^';
+            osResult += static_cast<char>( panInput[i] + '@' );
+        }
         else if( panInput[i] > 255 )
         {
             CPLString osUnicode;
@@ -404,7 +425,9 @@ CPLString OGRDXFWriterLayer::TextEscape( const char *pszInput )
             osResult += osUnicode;
         }
         else
+        {
             osResult += (char) panInput[i];
+        }
     }
 
     CPLFree(panInput);
@@ -412,6 +435,41 @@ CPLString OGRDXFWriterLayer::TextEscape( const char *pszInput )
     return osResult;
 }
 
+/************************************************************************/
+/*                     PrepareTextStyleDefinition()                     */
+/************************************************************************/
+std::map<CPLString, CPLString>
+OGRDXFWriterLayer::PrepareTextStyleDefinition( OGRStyleLabel *poLabelTool )
+{
+    GBool bDefault;
+
+    std::map<CPLString, CPLString> oTextStyleDef;
+
+/* -------------------------------------------------------------------- */
+/*      Fetch the data for this text style.                             */
+/* -------------------------------------------------------------------- */
+    const char *pszFontName = poLabelTool->FontName(bDefault);
+    if( !bDefault )
+        oTextStyleDef["Font"] = pszFontName;
+
+    const GBool bBold = poLabelTool->Bold(bDefault);
+    if( !bDefault )
+        oTextStyleDef["Bold"] = bBold ? "1" : "0";
+
+    const GBool bItalic = poLabelTool->Italic(bDefault);
+    if( !bDefault )
+        oTextStyleDef["Italic"] = bItalic ? "1" : "0";
+
+    const double dfStretch = poLabelTool->Stretch(bDefault);
+    if( !bDefault )
+    {
+        oTextStyleDef["Width"] = CPLString().Printf( "%f",
+            dfStretch / 100.0 );
+    }
+
+    return oTextStyleDef;
+}
+
 /************************************************************************/
 /*                             WriteTEXT()                              */
 /************************************************************************/
@@ -427,10 +485,10 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /* -------------------------------------------------------------------- */
 /*      Do we have styling information?                                 */
 /* -------------------------------------------------------------------- */
-    OGRStyleTool *poTool = NULL;
+    OGRStyleTool *poTool = nullptr;
     OGRStyleMgr oSM;
 
-    if( poFeature->GetStyleString() != NULL )
+    if( poFeature->GetStyleString() != nullptr )
     {
         oSM.InitFromFeature( poFeature );
 
@@ -441,6 +499,9 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /* ==================================================================== */
 /*      Process the LABEL tool.                                         */
 /* ==================================================================== */
+    double dfDx = 0.0;
+    double dfDy = 0.0;
+
     if( poTool && poTool->GetType() == OGRSTCLabel )
     {
         OGRStyleLabel *poLabel = (OGRStyleLabel *) poTool;
@@ -449,7 +510,7 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /* -------------------------------------------------------------------- */
 /*      Color                                                           */
 /* -------------------------------------------------------------------- */
-        if( poLabel->ForeColor(bDefault) != NULL && !bDefault )
+        if( poLabel->ForeColor(bDefault) != nullptr && !bDefault )
             WriteValue( 62, ColorStringToDXFColor(
                             poLabel->ForeColor(bDefault) ) );
 
@@ -458,8 +519,6 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /* -------------------------------------------------------------------- */
         const double dfAngle = poLabel->Angle(bDefault);
 
-        // The DXF2000 reference says this is in radians, but in files
-        // I see it seems to be in degrees. Perhaps this is version dependent?
         if( !bDefault )
             WriteValue( 50, dfAngle );
 
@@ -487,16 +546,57 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
                 WriteValue( 71, anAnchorMap[nAnchor] );
         }
 
+/* -------------------------------------------------------------------- */
+/*      Offset                                                          */
+/* -------------------------------------------------------------------- */
+        dfDx = poLabel->SpacingX(bDefault);
+        dfDy = poLabel->SpacingY(bDefault);
+
 /* -------------------------------------------------------------------- */
 /*      Escape the text, and convert to ISO8859.                        */
 /* -------------------------------------------------------------------- */
         const char *pszText = poLabel->TextString( bDefault );
 
-        if( pszText != NULL && !bDefault )
+        if( pszText != nullptr && !bDefault )
         {
             CPLString osEscaped = TextEscape( pszText );
+            while( osEscaped.size() > 250 )
+            {
+                WriteValue( 3, osEscaped.substr( 0, 250 ).c_str() );
+                osEscaped.erase( 0, 250 );
+            }
             WriteValue( 1, osEscaped );
         }
+
+/* -------------------------------------------------------------------- */
+/*      Store the text style in the map.                                */
+/* -------------------------------------------------------------------- */
+        std::map<CPLString, CPLString> oTextStyleDef =
+            PrepareTextStyleDefinition( poLabel );
+        CPLString osStyleName;
+
+        for( const auto& oPair: oNewTextStyles )
+        {
+            if( oPair.second == oTextStyleDef )
+            {
+                osStyleName = oPair.first;
+                break;
+            }
+        }
+
+        if( osStyleName == "" )
+        {
+
+            do
+            {
+                osStyleName.Printf( "AutoTextStyle-%d", nNextAutoID++ );
+            }
+            while( poDS->oHeaderDS.TextStyleExists( osStyleName ) );
+
+            oNewTextStyles[osStyleName] = oTextStyleDef;
+        }
+
+        WriteValue( 7, osStyleName );
     }
 
     delete poTool;
@@ -504,10 +604,10 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /* -------------------------------------------------------------------- */
 /*      Write the location.                                             */
 /* -------------------------------------------------------------------- */
-    OGRPoint *poPoint = (OGRPoint *) poFeature->GetGeometryRef();
+    OGRPoint *poPoint = poFeature->GetGeometryRef()->toPoint();
 
-    WriteValue( 10, poPoint->getX() );
-    if( !WriteValue( 20, poPoint->getY() ) )
+    WriteValue( 10, poPoint->getX() + dfDx );
+    if( !WriteValue( 20, poPoint->getY() + dfDy ) )
         return OGRERR_FAILURE;
 
     if( poPoint->getGeometryType() == wkbPoint25D )
@@ -522,28 +622,26 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /************************************************************************/
 /*                     PrepareLineTypeDefinition()                      */
 /************************************************************************/
-CPLString
-OGRDXFWriterLayer::PrepareLineTypeDefinition( CPL_UNUSED OGRFeature *poFeature,
-                                              OGRStyleTool *poTool )
+std::vector<double>
+OGRDXFWriterLayer::PrepareLineTypeDefinition( OGRStylePen *poPen )
 {
-    OGRStylePen *poPen = (OGRStylePen *) poTool;
-    GBool  bDefault;
 
 /* -------------------------------------------------------------------- */
 /*      Fetch pattern.                                                  */
 /* -------------------------------------------------------------------- */
+    GBool bDefault;
     const char *pszPattern = poPen->Pattern( bDefault );
+
     if( bDefault || strlen(pszPattern) == 0 )
-        return "";
+        return std::vector<double>();
 
 /* -------------------------------------------------------------------- */
 /*      Split into pen up / pen down bits.                              */
 /* -------------------------------------------------------------------- */
     char **papszTokens = CSLTokenizeString(pszPattern);
-    double dfTotalLength = 0;
-    CPLString osDef;
+    std::vector<double> adfWeightTokens;
 
-    for( int i = 0; papszTokens != NULL && papszTokens[i] != NULL; i++ )
+    for( int i = 0; papszTokens != nullptr && papszTokens[i] != nullptr; i++ )
     {
         const char *pszToken = papszTokens[i];
         CPLString osAmount;
@@ -552,7 +650,7 @@ OGRDXFWriterLayer::PrepareLineTypeDefinition( CPL_UNUSED OGRFeature *poFeature,
         // Split amount and unit.
         const char *pszUnit = pszToken;  // Used after for.
         for( ;
-             strchr( "0123456789.", *pszUnit) != NULL;
+             strchr( "0123456789.", *pszUnit) != nullptr;
              pszUnit++ ) {}
 
         osAmount.assign(pszToken,(int) (pszUnit-pszToken));
@@ -560,30 +658,39 @@ OGRDXFWriterLayer::PrepareLineTypeDefinition( CPL_UNUSED OGRFeature *poFeature,
         // If the unit is other than 'g' we really should be trying to
         // do some type of transformation - but what to do?  Pretty hard.
 
-        // Even entries are "pen down" represented as negative in DXF.
+        // Even entries are "pen down" represented as positive in DXF.
+        // "Pen up" entries (gaps) are represented as negative.
         if( i%2 == 0 )
-            osDXFEntry.Printf( " 49\n-%s\n 74\n0\n", osAmount.c_str() );
+            adfWeightTokens.push_back( CPLAtof( osAmount ) );
         else
-            osDXFEntry.Printf( " 49\n%s\n 74\n0\n", osAmount.c_str() );
-
-        osDef += osDXFEntry;
-
-        dfTotalLength += CPLAtof(osAmount);
+            adfWeightTokens.push_back( -CPLAtof( osAmount ) );
     }
 
-/* -------------------------------------------------------------------- */
-/*      Prefix 73 and 40 items to the definition.                       */
-/* -------------------------------------------------------------------- */
-    CPLString osPrefix;
-
-    osPrefix.Printf( " 73\n%d\n 40\n%.6g\n",
-                     CSLCount(papszTokens),
-                     dfTotalLength );
-    osDef = osPrefix + osDef;
-
     CSLDestroy( papszTokens );
 
-    return osDef;
+    return adfWeightTokens;
+}
+
+/************************************************************************/
+/*                       IsLineTypeProportional()                       */
+/************************************************************************/
+
+static double IsLineTypeProportional( const std::vector<double>& adfA,
+    const std::vector<double>& adfB )
+{
+    // If they are not the same length, they are not the same linetype
+    if( adfA.size() != adfB.size() )
+        return 0.0;
+
+    // Determine the proportion of the first elements
+    const double dfRatio = ( adfA[0] != 0.0 ) ? ( adfB[0] / adfA[0] ) : 0.0;
+
+    // Check if all elements follow this proportionality
+    for( size_t iIndex = 1; iIndex < adfA.size(); iIndex++ )
+        if( fabs( adfB[iIndex] - ( adfA[iIndex] * dfRatio ) ) > 1e-6 )
+            return 0.0;
+
+    return dfRatio;
 }
 
 /************************************************************************/
@@ -591,14 +698,14 @@ OGRDXFWriterLayer::PrepareLineTypeDefinition( CPL_UNUSED OGRFeature *poFeature,
 /************************************************************************/
 
 OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
-                                         OGRGeometry *poGeom )
+                                         const OGRGeometry *poGeom )
 
 {
 /* -------------------------------------------------------------------- */
 /*      For now we handle multilinestrings by writing a series of       */
 /*      entities.                                                       */
 /* -------------------------------------------------------------------- */
-    if( poGeom == NULL )
+    if( poGeom == nullptr )
         poGeom = poFeature->GetGeometryRef();
 
     if ( poGeom->IsEmpty() )
@@ -609,14 +716,13 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
     if( wkbFlatten(poGeom->getGeometryType()) == wkbMultiPolygon
         || wkbFlatten(poGeom->getGeometryType()) == wkbMultiLineString )
     {
-        OGRGeometryCollection *poGC = (OGRGeometryCollection *) poGeom;
+        const OGRGeometryCollection *poGC = poGeom->toGeometryCollection();
         OGRErr eErr = OGRERR_NONE;
-
-        for( int iGeom = 0;
-             eErr == OGRERR_NONE && iGeom < poGC->getNumGeometries();
-             iGeom++ )
+        for( auto&& poMember: *poGC )
         {
-            eErr = WritePOLYLINE( poFeature, poGC->getGeometryRef( iGeom ) );
+            eErr = WritePOLYLINE( poFeature, poMember );
+            if( eErr != OGRERR_NONE )
+                break;
         }
 
         return eErr;
@@ -628,15 +734,13 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
     if( wkbFlatten(poGeom->getGeometryType()) == wkbPolygon
         || wkbFlatten(poGeom->getGeometryType()) == wkbTriangle)
     {
-        OGRPolygon *poPoly = (OGRPolygon *) poGeom;
-        OGRErr eErr;
-
-        eErr = WritePOLYLINE( poFeature, poPoly->getExteriorRing() );
-        for( int iGeom = 0;
-             eErr == OGRERR_NONE && iGeom < poPoly->getNumInteriorRings();
-             iGeom++ )
+        const OGRPolygon *poPoly = poGeom->toPolygon();
+        OGRErr eErr = OGRERR_NONE;
+        for( auto&& poRing: *poPoly )
         {
-            eErr = WritePOLYLINE( poFeature, poPoly->getInteriorRing(iGeom) );
+            eErr = WritePOLYLINE( poFeature, poRing );
+            if( eErr != OGRERR_NONE )
+                break;
         }
 
         return eErr;
@@ -648,7 +752,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
     if( wkbFlatten(poGeom->getGeometryType()) != wkbLineString )
         return OGRERR_UNSUPPORTED_GEOMETRY_TYPE;
 
-    OGRLineString *poLS = (OGRLineString *) poGeom;
+    const OGRLineString *poLS = poGeom->toLineString();
 
 /* -------------------------------------------------------------------- */
 /*      Write as a lightweight polygon,                                 */
@@ -692,10 +796,10 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
 /* -------------------------------------------------------------------- */
 /*      Do we have styling information?                                 */
 /* -------------------------------------------------------------------- */
-    OGRStyleTool *poTool = NULL;
+    OGRStyleTool *poTool = nullptr;
     OGRStyleMgr oSM;
 
-    if( poFeature->GetStyleString() != NULL )
+    if( poFeature->GetStyleString() != nullptr )
     {
         oSM.InitFromFeature( poFeature );
 
@@ -712,7 +816,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
         OGRStylePen *poPen = (OGRStylePen *) poTool;
         GBool bDefault;
 
-        if( poPen->Color(bDefault) != NULL && !bDefault )
+        if( poPen->Color(bDefault) != nullptr && !bDefault )
             WriteValue( 62, ColorStringToDXFColor( poPen->Color(bDefault) ) );
 
         // we want to fetch the width in ground units.
@@ -727,51 +831,102 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
 /*      Do we have a Linetype for the feature?                          */
 /* -------------------------------------------------------------------- */
     CPLString osLineType = poFeature->GetFieldAsString( "Linetype" );
+    double dfLineTypeScale = 0.0;
 
-    if( !osLineType.empty()
-        && (poDS->oHeaderDS.LookupLineType( osLineType ) != NULL
-            || oNewLineTypes.count(osLineType) > 0 ) )
+    bool bGotLinetype = false;
+
+    if( !osLineType.empty() )
     {
-        // Already define -> just reference it.
-        WriteValue( 6, osLineType );
+        std::vector<double> adfLineType =
+            poDS->oHeaderDS.LookupLineType( osLineType );
+
+        if( adfLineType.empty() && oNewLineTypes.count(osLineType) > 0 )
+            adfLineType = oNewLineTypes[osLineType];
+
+        if( !adfLineType.empty() )
+        {
+            bGotLinetype = true;
+            WriteValue( 6, osLineType );
+
+            // If the given linetype is proportional to the linetype data
+            // in the style string, then apply a linetype scale
+            if( poTool != nullptr && poTool->GetType() == OGRSTCPen )
+            {
+                std::vector<double> adfDefinition = PrepareLineTypeDefinition(
+                    static_cast<OGRStylePen *>( poTool ) );
+                
+                if( !adfDefinition.empty() )
+                {
+                    dfLineTypeScale = IsLineTypeProportional( adfLineType,
+                        adfDefinition );
+
+                    if( dfLineTypeScale != 0.0 && 
+                        fabs( dfLineTypeScale - 1.0 ) > 1e-4 )
+                    {
+                        WriteValue( 48, dfLineTypeScale );
+                    }
+                }
+            }
+        }
     }
-    else if( poTool != NULL && poTool->GetType() == OGRSTCPen )
-    {
-        CPLString osDefinition = PrepareLineTypeDefinition( poFeature,
-                                                            poTool );
 
-        if( osDefinition != "" && osLineType == "" )
+    if( !bGotLinetype && poTool != nullptr && poTool->GetType() == OGRSTCPen )
+    {
+        std::vector<double> adfDefinition = PrepareLineTypeDefinition(
+            static_cast<OGRStylePen *>( poTool ) );
+
+        if( !adfDefinition.empty() )
         {
             // Is this definition already created and named?
-            std::map<CPLString,CPLString>::iterator it;
-
-            for( it = oNewLineTypes.begin();
-                 it != oNewLineTypes.end();
-                 ++it )
+            for( const auto& oPair : poDS->oHeaderDS.GetLineTypeTable() )
             {
-                if( (*it).second == osDefinition )
+                dfLineTypeScale = IsLineTypeProportional( oPair.second,
+                    adfDefinition );
+                if( dfLineTypeScale != 0.0 )
                 {
-                    osLineType = (*it).first;
+                    osLineType = oPair.first;
                     break;
                 }
             }
 
-            // create an automatic name for it.
+            if( dfLineTypeScale == 0.0 )
+            {
+                for( const auto& oPair : oNewLineTypes )
+                {
+                    dfLineTypeScale = IsLineTypeProportional( oPair.second,
+                        adfDefinition );
+                    if( dfLineTypeScale != 0.0 )
+                    {
+                        osLineType = oPair.first;
+                        break;
+                    }
+                }
+            }
+
+            // If not, create an automatic name for it.
             if( osLineType == "" )
             {
+                dfLineTypeScale = 1.0;
                 do
                 {
                     osLineType.Printf( "AutoLineType-%d", nNextAutoID++ );
                 }
-                while( poDS->oHeaderDS.LookupLineType(osLineType) != NULL );
+                while( poDS->oHeaderDS.LookupLineType(osLineType).size() > 0 );
+            }
+
+            // If it isn't already defined, add it now.
+            if( poDS->oHeaderDS.LookupLineType( osLineType ).empty() &&
+                oNewLineTypes.count( osLineType ) == 0 )
+            {
+                oNewLineTypes[osLineType] = adfDefinition;
             }
-        }
 
-        // If it isn't already defined, add it now.
-        if( osDefinition != "" && oNewLineTypes.count(osLineType) == 0 )
-        {
-            oNewLineTypes[osLineType] = osDefinition;
             WriteValue( 6, osLineType );
+
+            if( dfLineTypeScale != 0.0 && fabs( dfLineTypeScale - 1.0 ) > 1e-4 )
+            {
+                WriteValue( 48, dfLineTypeScale );
+            }
         }
     }
 
@@ -867,7 +1022,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 /*      For now we handle multipolygons by writing a series of          */
 /*      entities.                                                       */
 /* -------------------------------------------------------------------- */
-    if( poGeom == NULL )
+    if( poGeom == nullptr )
         poGeom = poFeature->GetGeometryRef();
 
     if ( poGeom->IsEmpty() )
@@ -877,14 +1032,12 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 
     if( wkbFlatten(poGeom->getGeometryType()) == wkbMultiPolygon )
     {
-        OGRGeometryCollection *poGC = (OGRGeometryCollection *) poGeom;
         OGRErr eErr = OGRERR_NONE;
-
-        for( int iGeom = 0;
-             eErr == OGRERR_NONE && iGeom < poGC->getNumGeometries();
-             iGeom++ )
+        for( auto&& poMember: poGeom->toMultiPolygon() )
         {
-            eErr = WriteHATCH( poFeature, poGC->getGeometryRef( iGeom ) );
+            eErr = WriteHATCH( poFeature, poMember );
+            if( eErr != OGRERR_NONE )
+                break;
         }
 
         return eErr;
@@ -907,9 +1060,14 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
     WriteValue( 100, "AcDbEntity" );
     WriteValue( 100, "AcDbHatch" );
 
-    WriteValue( 10, 0 ); // elevation point X. 0 for DXF
-    WriteValue( 20, 0 ); // elevation point Y
-    WriteValue( 30, 0 ); // elevation point Z
+    // Figure out "average" elevation
+    OGREnvelope3D oEnv;
+    poGeom->getEnvelope( &oEnv );
+    WriteValue( 10, 0 ); // elevation point X = 0
+    WriteValue( 20, 0 ); // elevation point Y = 0
+    // elevation point Z = constant elevation
+    WriteValue( 30, oEnv.MinZ + ( oEnv.MaxZ - oEnv.MinZ ) / 2 );
+
     WriteValue(210, 0 ); // extrusion direction X
     WriteValue(220, 0 ); // extrusion direction Y
     WriteValue(230,1.0); // extrusion direction Z
@@ -921,10 +1079,10 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 /* -------------------------------------------------------------------- */
 /*      Do we have styling information?                                 */
 /* -------------------------------------------------------------------- */
-    OGRStyleTool *poTool = NULL;
+    OGRStyleTool *poTool = nullptr;
     OGRStyleMgr oSM;
 
-    if( poFeature->GetStyleString() != NULL )
+    if( poFeature->GetStyleString() != nullptr )
     {
         oSM.InitFromFeature( poFeature );
 
@@ -937,7 +1095,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
         OGRStyleBrush *poBrush = (OGRStyleBrush *) poTool;
         GBool  bDefault;
 
-        if( poBrush->ForeColor(bDefault) != NULL && !bDefault )
+        if( poBrush->ForeColor(bDefault) != nullptr && !bDefault )
             WriteValue( 62, ColorStringToDXFColor( poBrush->ForeColor(bDefault) ) );
     }
     delete poTool;
@@ -1018,16 +1176,12 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 /* -------------------------------------------------------------------- */
 /*      Process the loops (rings).                                      */
 /* -------------------------------------------------------------------- */
-    OGRPolygon *poPoly = (OGRPolygon *) poGeom;
+    OGRPolygon *poPoly = poGeom->toPolygon();
 
     WriteValue( 91, poPoly->getNumInteriorRings() + 1 );
 
-    for( int iRing = -1; iRing < poPoly->getNumInteriorRings(); iRing++ )
+    for( auto&& poLR: *poPoly )
     {
-        OGRLinearRing *poLR = iRing == -1
-            ? poPoly->getExteriorRing()
-            : poPoly->getInteriorRing( iRing );
-
         WriteValue( 92, 2 ); // Polyline
         WriteValue( 72, 0 ); // has bulge
         WriteValue( 73, 1 ); // is closed
@@ -1094,7 +1248,7 @@ OGRErr OGRDXFWriterLayer::ICreateFeature( OGRFeature *poFeature )
     OGRGeometry *poGeom = poFeature->GetGeometryRef();
     OGRwkbGeometryType eGType = wkbNone;
 
-    if( poGeom != NULL )
+    if( poGeom != nullptr )
     {
         if( !poGeom->IsEmpty() )
         {
@@ -1109,25 +1263,19 @@ OGRErr OGRDXFWriterLayer::ICreateFeature( OGRFeature *poFeature )
     {
         const char *pszBlockName = poFeature->GetFieldAsString("BlockName");
 
-        // we don't want to treat as a block ref if we are writing blocks layer
-        if( pszBlockName != NULL
-            && poDS->poBlocksLayer != NULL
-            && poFeature->GetDefnRef() == poDS->poBlocksLayer->GetLayerDefn())
-            pszBlockName = NULL;
-
         // We don't want to treat as a blocks ref if the block is not defined
         if( pszBlockName
-            && poDS->oHeaderDS.LookupBlock(pszBlockName) == NULL )
+            && poDS->oHeaderDS.LookupBlock(pszBlockName) == nullptr )
         {
-            if( poDS->poBlocksLayer == NULL
-                || poDS->poBlocksLayer->FindBlock(pszBlockName) == NULL )
-                pszBlockName = NULL;
+            if( poDS->poBlocksLayer == nullptr
+                || poDS->poBlocksLayer->FindBlock(pszBlockName) == nullptr )
+                pszBlockName = nullptr;
         }
 
-        if( pszBlockName != NULL )
+        if( pszBlockName != nullptr )
             return WriteINSERT( poFeature );
 
-        else if( poFeature->GetStyleString() != NULL
+        else if( poFeature->GetStyleString() != nullptr
             && STARTS_WITH_CI(poFeature->GetStyleString(), "LABEL") )
             return WriteTEXT( poFeature );
         else
@@ -1150,16 +1298,19 @@ OGRErr OGRDXFWriterLayer::ICreateFeature( OGRFeature *poFeature )
     // Explode geometry collections into multiple entities.
     else if( eGType == wkbGeometryCollection )
     {
-        OGRGeometryCollection *poGC = (OGRGeometryCollection *)
-            poFeature->StealGeometry();
-        for( int iGeom = 0; iGeom < poGC->getNumGeometries(); iGeom++ )
+        OGRGeometryCollection *poGC =
+            poFeature->StealGeometry()->toGeometryCollection();
+        for( auto&& poMember: poGC )
         {
-            poFeature->SetGeometry( poGC->getGeometryRef(iGeom) );
+            poFeature->SetGeometry( poMember );
 
             OGRErr eErr = CreateFeature( poFeature );
 
             if( eErr != OGRERR_NONE )
+            {
+                delete poGC;
                 return eErr;
+            }
         }
 
         poFeature->SetGeometryDirectly( poGC );
@@ -1184,7 +1335,7 @@ int OGRDXFWriterLayer::ColorStringToDXFColor( const char *pszRGB )
 /* -------------------------------------------------------------------- */
 /*      Parse the RGB string.                                           */
 /* -------------------------------------------------------------------- */
-    if( pszRGB == NULL )
+    if( pszRGB == nullptr )
         return -1;
 
     unsigned int nRed = 0;
