@@ -35,7 +35,7 @@
 #include "ogr_p.h"
 #include "cpl_vsi_error.h"
 
-CPL_CVSID("$Id: ogrlibkmldatasource.cpp 25d9b4bdc60ff910a93d9de2d454f4aef4c9dbdc 2018-03-09 19:57:59Z Even Rouault $")
+CPL_CVSID("$Id: ogrlibkmldatasource.cpp 2957de2c8bc039c0b96299bffec0ddea3be22772 2018-10-06 21:53:20 +0200 Even Rouault $")
 
 using kmlbase::Attributes;
 using kmldom::ContainerPtr;
@@ -71,16 +71,25 @@ static const char OGRLIBKMLSRSWKT[] =
 /*                           OGRLIBKMLParse()                           */
 /************************************************************************/
 
-static ElementPtr OGRLIBKMLParse(std::string oKml, std::string *posError)
+static ElementPtr OGRLIBKMLParse(const std::string& oKml, std::string *posError)
 {
-    // To allow reading files using an explicit namespace prefix like <kml:kml>
-    // we need to use ParseNS (see #6981). But if we use ParseNS, we have
-    // issues reading gx: elements. So use ParseNS only when we have errors
-    // with Parse. This is not completely satisfactory...
-    ElementPtr element = kmldom::Parse(oKml, posError);
-    if( !element )
-        element = kmldom::ParseNS(oKml, posError);
-    return element;
+    try
+    {
+        // To allow reading files using an explicit namespace prefix like <kml:kml>
+        // we need to use ParseNS (see #6981). But if we use ParseNS, we have
+        // issues reading gx: elements. So use ParseNS only when we have errors
+        // with Parse. This is not completely satisfactory...
+        ElementPtr element = kmldom::Parse(oKml, posError);
+        if( !element )
+            element = kmldom::ParseNS(oKml, posError);
+        return element;
+    }
+    catch(const std::exception& ex)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "LIBKML: libstdc++ exception during Parse() : %s", ex.what());
+        return nullptr;
+    }
 }
 
 /******************************************************************************
@@ -163,9 +172,10 @@ static void OGRLIBKMLPreProcessInput( std::string& oKml )
         bool bDigitFound = false;
         for( ; nPos < nPosEnd; nPos++ )
         {
-            if( oKml[nPos] >= '0' && oKml[nPos] <= '9' )
+            char ch = oKml[nPos];
+            if( ch >= '0' && ch <= '9' )
                 bDigitFound = true;
-            else if( oKml[nPos] == '\t' )
+            else if( ch == '\t' || ch == '\n' )
                 oKml[nPos] = ' ';
         }
         if( !bDigitFound )
@@ -1011,7 +1021,8 @@ int OGRLIBKMLDataSource::ParseIntoStyleTable(
 int OGRLIBKMLDataSource::OpenKml( const char *pszFilename, int bUpdateIn )
 {
     std::string oKmlKml;
-    char szBuffer[1024+1] = {};
+    std::string osBuffer;
+    osBuffer.resize(4096);
 
     VSILFILE* fp = VSIFOpenL(pszFilename, "rb");
     if( fp == nullptr )
@@ -1021,14 +1032,17 @@ int OGRLIBKMLDataSource::OpenKml( const char *pszFilename, int bUpdateIn )
         return FALSE;
     }
     int nRead = 0;
-    while( (nRead = static_cast<int>(VSIFReadL(szBuffer, 1, 1024, fp))) != 0 )
+    while( (nRead = static_cast<int>(VSIFReadL(&osBuffer[0], 1,
+                                               osBuffer.size(), fp))) != 0 )
     {
         try
         {
-            oKmlKml.append(szBuffer, nRead);
+            oKmlKml.append(osBuffer.c_str(), nRead);
         }
-        catch( const std::bad_alloc& )
+        catch(const std::exception& ex)
         {
+            CPLDebug("LIBKML",
+                 "libstdc++ exception during ingestion: %s", ex.what());
             VSIFCloseL(fp);
             return FALSE;
         }
@@ -1045,9 +1059,9 @@ int OGRLIBKMLDataSource::OpenKml( const char *pszFilename, int bUpdateIn )
     /***** parse the kml into the DOM *****/
     std::string oKmlErrors;
 
-    ElementPtr poKmlRoot = OGRLIBKMLParse( oKmlKml, &oKmlErrors );
+    m_poKmlDSKml = AsKml(OGRLIBKMLParse( oKmlKml, &oKmlErrors ));
 
-    if( !poKmlRoot )
+    if( !m_poKmlDSKml )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "ERROR parsing kml %s :%s",
@@ -1059,7 +1073,7 @@ int OGRLIBKMLDataSource::OpenKml( const char *pszFilename, int bUpdateIn )
 
     /***** get the container from root  *****/
     if( !( m_poKmlDSContainer = GetContainerFromRoot( m_poKmlFactory,
-                                                      poKmlRoot ) ) )
+                                                      m_poKmlDSKml ) ) )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "ERROR parsing kml %s :%s %s",
@@ -1090,7 +1104,7 @@ int OGRLIBKMLDataSource::OpenKml( const char *pszFilename, int bUpdateIn )
 
       AddLayer( layername_default.c_str(),
                 wkbUnknown,
-                this, poKmlRoot, m_poKmlDSContainer, pszFilename, FALSE,
+                this, m_poKmlDSKml, m_poKmlDSContainer, pszFilename, FALSE,
                 bUpdateIn, 1 );
     }
 
