@@ -31,9 +31,9 @@
 #include "cpl_string.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrpgdumplayer.cpp 38135 2017-04-25 22:42:55Z rouault $");
+CPL_CVSID("$Id: ogrpgdumplayer.cpp 1dcdb5c8f617002a6e034d4d98dfdf080f709325 2018-01-14 01:50:09Z Kurt Schwehr $")
 
-static const int USE_COPY_UNSET = -1;
+constexpr int USE_COPY_UNSET = -1;
 
 static CPLString OGRPGDumpEscapeStringList(
     char** papszItems, bool bForInsertOrUpdate,
@@ -84,7 +84,8 @@ OGRPGDumpLayer::OGRPGDumpLayer( OGRPGDumpDataSource* poDSIn,
     iFIDAsRegularColumnIndex(-1),
     bAutoFIDOnCreateViaCopy(true),
     bCopyStatementWithFID(false),
-    papszOverrideColumnTypes(NULL)
+    bNeedToUpdateSequence(false),
+    papszOverrideColumnTypes(nullptr)
 {
     SetDescription( poFeatureDefn->GetName() );
     poFeatureDefn->SetGeomType(wkbNone);
@@ -98,6 +99,7 @@ OGRPGDumpLayer::OGRPGDumpLayer( OGRPGDumpDataSource* poDSIn,
 OGRPGDumpLayer::~OGRPGDumpLayer()
 {
     EndCopy();
+    UpdateSequenceIfNeeded();
 
     poFeatureDefn->Release();
     CPLFree(pszSchemaName);
@@ -113,7 +115,7 @@ OGRPGDumpLayer::~OGRPGDumpLayer()
 OGRFeature *OGRPGDumpLayer::GetNextFeature()
 {
     CPLError(CE_Failure, CPLE_NotSupported, "PGDump driver is write only");
-    return NULL;
+    return nullptr;
 }
 
 /************************************************************************/
@@ -138,7 +140,7 @@ int OGRPGDumpLayer::TestCapability( const char * pszCap )
 
 OGRErr OGRPGDumpLayer::ICreateFeature( OGRFeature *poFeature )
 {
-    if( NULL == poFeature )
+    if( nullptr == poFeature )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "NULL pointer to OGRFeature passed to CreateFeature()." );
@@ -190,7 +192,7 @@ OGRErr OGRPGDumpLayer::ICreateFeature( OGRFeature *poFeature )
         for( int iField = 0; iField < nFieldCount; iField++ )
         {
             if( !poFeature->IsFieldSetAndNotNull( iField ) &&
-                poFeature->GetFieldDefnRef(iField)->GetDefault() != NULL )
+                poFeature->GetFieldDefnRef(iField)->GetDefault() != nullptr )
             {
                 bHasDefaultValue = true;
                 break;
@@ -220,6 +222,7 @@ OGRErr OGRPGDumpLayer::ICreateFeature( OGRFeature *poFeature )
                     // FID column is an autoincremented column.
                     StartCopy(bFIDSet);
                     bCopyStatementWithFID = bFIDSet;
+                    bNeedToUpdateSequence = bFIDSet;
                 }
 
                 eErr = CreateFeatureViaCopy( poFeature );
@@ -249,7 +252,7 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
 {
     OGRErr eErr = OGRERR_FAILURE;
 
-    if( NULL == poFeature )
+    if( nullptr == poFeature )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "NULL pointer to OGRFeature passed to CreateFeatureViaInsert()." );
@@ -267,7 +270,7 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
     for( int i = 0; i < poFeatureDefn->GetGeomFieldCount(); i++ )
     {
         OGRGeometry *poGeom = poFeature->GetGeomFieldRef(i);
-        if( poGeom != NULL )
+        if( poGeom != nullptr )
         {
             if( bNeedComma )
                 osCommand += ", ";
@@ -278,14 +281,19 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
         }
     }
 
-    if( poFeature->GetFID() != OGRNullFID && pszFIDColumn != NULL )
+    if( poFeature->GetFID() != OGRNullFID && pszFIDColumn != nullptr )
     {
+        bNeedToUpdateSequence = true;
         if( bNeedComma )
             osCommand += ", ";
 
         osCommand = osCommand + OGRPGDumpEscapeColumnName(pszFIDColumn) + " ";
         bNeedComma = true;
     }
+    else
+    {
+        UpdateSequenceIfNeeded();
+    }
 
     for( int i = 0; i < poFeatureDefn->GetFieldCount(); i++ )
     {
@@ -312,9 +320,9 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
     for( int i = 0; i < poFeatureDefn->GetGeomFieldCount(); i++ )
     {
         OGRGeometry *poGeom = poFeature->GetGeomFieldRef(i);
-        if( poGeom != NULL )
+        if( poGeom != nullptr )
         {
-            char *pszWKT = NULL;
+            char *pszWKT = nullptr;
 
             OGRPGDumpGeomFieldDefn* poGFldDefn =
                 (OGRPGDumpGeomFieldDefn*) poFeature->GetGeomFieldDefnRef(i);
@@ -341,7 +349,7 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
             {
                 poGeom->exportToWkt( &pszWKT );
 
-                if( pszWKT != NULL )
+                if( pszWKT != nullptr )
                 {
                     osCommand +=
                         CPLString().Printf(
@@ -357,7 +365,7 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
     }
 
     /* Set the FID */
-    if( poFeature->GetFID() != OGRNullFID && pszFIDColumn != NULL )
+    if( poFeature->GetFID() != OGRNullFID && pszFIDColumn != nullptr )
     {
         if( bNeedComma )
             osCommand += ", ";
@@ -378,7 +386,7 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
             bNeedComma = true;
 
         OGRPGCommonAppendFieldValue(osCommand, poFeature, i,
-                                    OGRPGDumpEscapeStringWithUserData, NULL);
+                                    OGRPGDumpEscapeStringWithUserData, nullptr);
     }
 
     osCommand += ")";
@@ -409,8 +417,8 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaCopy( OGRFeature *poFeature )
     for( int i = 0; i < poFeature->GetGeomFieldCount(); i++ )
     {
         OGRGeometry *poGeometry = poFeature->GetGeomFieldRef(i);
-        char *pszGeom = NULL;
-        if ( NULL != poGeometry /* && (bHasWkb || bHasPostGISGeometry || bHasPostGISGeography) */)
+        char *pszGeom = nullptr;
+        if ( nullptr != poGeometry /* && (bHasWkb || bHasPostGISGeometry || bHasPostGISGeography) */)
         {
             OGRPGDumpGeomFieldDefn* poGFldDefn =
                 (OGRPGDumpGeomFieldDefn*) poFeature->GetGeomFieldDefnRef(i);
@@ -447,7 +455,7 @@ OGRErr OGRPGDumpLayer::CreateFeatureViaCopy( OGRFeature *poFeature )
                                           pszFIDColumn,
                                           bFIDColumnInCopyFields,
                                           OGRPGDumpEscapeStringWithUserData,
-                                          NULL);
+                                          nullptr);
 
     /* Add end of line marker */
     // osCommand += "\n";
@@ -508,7 +516,7 @@ void OGRPGCommonAppendCopyFieldsExceptGeom(
             continue;
 
         const char *pszStrValue = poFeature->GetFieldAsString(i);
-        char *pszNeedToFree = NULL;
+        char *pszNeedToFree = nullptr;
 
         if( bAddTab )
             osCommand += "\t";
@@ -717,9 +725,31 @@ OGRErr OGRPGDumpLayer::EndCopy()
 
     bUseCopy = USE_COPY_UNSET;
 
+    UpdateSequenceIfNeeded();
+
     return OGRERR_NONE;
 }
 
+/************************************************************************/
+/*                       UpdateSequenceIfNeeded()                       */
+/************************************************************************/
+
+void OGRPGDumpLayer::UpdateSequenceIfNeeded()
+{
+    if( bNeedToUpdateSequence && pszFIDColumn != nullptr )
+    {
+        CPLString osCommand;
+        osCommand.Printf(
+            "SELECT setval(pg_get_serial_sequence(%s, %s), MAX(%s)) FROM %s",
+            OGRPGDumpEscapeString(pszSqlTableName).c_str(),
+            OGRPGDumpEscapeString(pszFIDColumn).c_str(),
+            OGRPGDumpEscapeColumnName(pszFIDColumn).c_str(),
+            pszSqlTableName);
+        poDS->Log(osCommand);
+        bNeedToUpdateSequence = false;
+    }
+}
+
 /************************************************************************/
 /*                          BuildCopyFields()                           */
 /************************************************************************/
@@ -739,7 +769,7 @@ CPLString OGRPGDumpLayer::BuildCopyFields( int bSetFID )
     }
 
     int nFIDIndex = -1;
-    bFIDColumnInCopyFields = pszFIDColumn != NULL && bSetFID;
+    bFIDColumnInCopyFields = pszFIDColumn != nullptr && bSetFID;
     if( bFIDColumnInCopyFields )
     {
         if( !osFieldList.empty() )
@@ -931,7 +961,7 @@ static CPLString OGRPGDumpEscapeStringList(
     if (bForInsertOrUpdate)
     {
         osStr += "]";
-        if( papszItems == NULL )
+        if( papszItems == nullptr )
             osStr += "::varchar[]";
     }
     else
@@ -966,10 +996,9 @@ void OGRPGCommonAppendFieldValue(CPLString& osCommand,
     {
         int nCount, nOff = 0, j;
         const int *panItems = poFeature->GetFieldAsIntegerList(i,&nCount);
-        char *pszNeedToFree = NULL;
 
         const size_t nLen = nCount * 13 + 10;
-        pszNeedToFree = (char *) CPLMalloc(nLen);
+        char *pszNeedToFree = (char *) CPLMalloc(nLen);
         strcpy( pszNeedToFree, "'{" );
         for( j = 0; j < nCount; j++ )
         {
@@ -991,10 +1020,9 @@ void OGRPGCommonAppendFieldValue(CPLString& osCommand,
     {
         int nCount, nOff = 0, j;
         const GIntBig *panItems = poFeature->GetFieldAsInteger64List(i,&nCount);
-        char *pszNeedToFree = NULL;
 
         const size_t nLen = nCount * 26 + 10;
-        pszNeedToFree = (char *) CPLMalloc(nLen);
+        char *pszNeedToFree = (char *) CPLMalloc(nLen);
         strcpy( pszNeedToFree, "'{" );
         for( j = 0; j < nCount; j++ )
         {
@@ -1018,10 +1046,9 @@ void OGRPGCommonAppendFieldValue(CPLString& osCommand,
         int nCount = 0;
         int nOff = 0;
         const double *padfItems = poFeature->GetFieldAsDoubleList(i,&nCount);
-        char *pszNeedToFree = NULL;
 
         const size_t nLen = nCount * 40 + 10;
-        pszNeedToFree = (char *) CPLMalloc(nLen);
+        char *pszNeedToFree = (char *) CPLMalloc(nLen);
         strcpy( pszNeedToFree, "'{" );
         for( int j = 0; j < nCount; j++ )
         {
@@ -1303,7 +1330,7 @@ bool OGRPGCommonLayerSetType( OGRFieldDefn& oField,
             int    nPrecision = 0;
 
             nWidth = atoi(pszFormatType + 8);
-            if( pszPrecision != NULL )
+            if( pszPrecision != nullptr )
                 nPrecision = atoi(pszPrecision+1);
 
             if( nPrecision == 0 )
@@ -1330,7 +1357,7 @@ bool OGRPGCommonLayerSetType( OGRFieldDefn& oField,
             int    nPrecision = 0;
 
             nWidth = atoi(pszFormatType + 8);
-            if( pszPrecision != NULL )
+            if( pszPrecision != nullptr )
                 nPrecision = atoi(pszPrecision+1);
 
             if( nPrecision == 0 )
@@ -1432,7 +1459,7 @@ bool OGRPGCommonLayerSetType( OGRFieldDefn& oField,
 void OGRPGCommonLayerNormalizeDefault(OGRFieldDefn* poFieldDefn,
                                       const char* pszDefault)
 {
-    if(pszDefault==NULL)
+    if(pszDefault==nullptr)
         return;
     CPLString osDefault(pszDefault);
     size_t nPos = osDefault.find("::character varying");
@@ -1525,7 +1552,7 @@ OGRErr OGRPGDumpLayer::CreateField( OGRFieldDefn *poFieldIn,
         CPLTestBool(CPLGetConfigOption(
             "PGDUMP_DEBUG_ALLOW_CREATION_FIELD_WITH_FID_NAME", "YES"));
 
-    if( bAllowCreationOfFieldWithFIDName && pszFIDColumn != NULL &&
+    if( bAllowCreationOfFieldWithFIDName && pszFIDColumn != nullptr &&
         EQUAL( oField.GetNameRef(), pszFIDColumn ) &&
         oField.GetType() != OFTInteger &&
         oField.GetType() != OFTInteger64 )
@@ -1558,7 +1585,7 @@ OGRErr OGRPGDumpLayer::CreateField( OGRFieldDefn *poFieldIn,
 
     const char* pszOverrideType =
         CSLFetchNameValue(papszOverrideColumnTypes, oField.GetNameRef());
-    if( pszOverrideType != NULL )
+    if( pszOverrideType != nullptr )
     {
         osFieldType = pszOverrideType;
     }
@@ -1581,7 +1608,7 @@ OGRErr OGRPGDumpLayer::CreateField( OGRFieldDefn *poFieldIn,
                       osFieldType.c_str() );
     if( !oField.IsNullable() )
         osCommand += " NOT NULL";
-    if( oField.GetDefault() != NULL && !oField.IsDefaultDriverSpecific() )
+    if( oField.GetDefault() != nullptr && !oField.IsDefaultDriverSpecific() )
     {
         osCommand += " DEFAULT ";
         osCommand += OGRPGCommonLayerGetPGDefault(&oField);
@@ -1589,7 +1616,7 @@ OGRErr OGRPGDumpLayer::CreateField( OGRFieldDefn *poFieldIn,
 
     poFeatureDefn->AddFieldDefn( &oField );
 
-    if( bAllowCreationOfFieldWithFIDName && pszFIDColumn != NULL &&
+    if( bAllowCreationOfFieldWithFIDName && pszFIDColumn != nullptr &&
         EQUAL( oField.GetNameRef(), pszFIDColumn ) )
     {
         iFIDAsRegularColumnIndex = poFeatureDefn->GetFieldCount() - 1;
@@ -1651,18 +1678,18 @@ OGRErr OGRPGDumpLayer::CreateGeomField( OGRGeomFieldDefn *poGeomFieldIn,
     int nSRSId = nUnknownSRSId;
     if( nForcedSRSId != -2 )
         nSRSId = nForcedSRSId;
-    else if( poSRS != NULL )
+    else if( poSRS != nullptr )
     {
-        const char* pszAuthorityName = poSRS->GetAuthorityName(NULL);
-        if( pszAuthorityName != NULL && EQUAL( pszAuthorityName, "EPSG" ) )
+        const char* pszAuthorityName = poSRS->GetAuthorityName(nullptr);
+        if( pszAuthorityName != nullptr && EQUAL( pszAuthorityName, "EPSG" ) )
         {
             /* Assume the EPSG Id is the SRS ID. Might be a wrong guess ! */
-            nSRSId = atoi( poSRS->GetAuthorityCode(NULL) );
+            nSRSId = atoi( poSRS->GetAuthorityCode(nullptr) );
         }
         else
         {
             const char* pszGeogCSName = poSRS->GetAttrValue("GEOGCS");
-            if (pszGeogCSName != NULL && EQUAL(pszGeogCSName, "GCS_WGS_1984"))
+            if (pszGeogCSName != nullptr && EQUAL(pszGeogCSName, "GCS_WGS_1984"))
                 nSRSId = 4326;
         }
     }
@@ -1744,7 +1771,7 @@ OGRErr OGRPGDumpLayer::CreateGeomField( OGRGeomFieldDefn *poGeomFieldIn,
 
 void OGRPGDumpLayer::SetOverrideColumnTypes( const char* pszOverrideColumnTypes )
 {
-    if( pszOverrideColumnTypes == NULL )
+    if( pszOverrideColumnTypes == nullptr )
         return;
 
     const char* pszIter = pszOverrideColumnTypes;
@@ -1790,12 +1817,12 @@ CPLErr OGRPGDumpLayer::SetMetadata(char** papszMD, const char* pszDomain)
 {
     OGRLayer::SetMetadata(papszMD, pszDomain);
     if( !osForcedDescription.empty() &&
-        (pszDomain == NULL || EQUAL(pszDomain, "")) )
+        (pszDomain == nullptr || EQUAL(pszDomain, "")) )
     {
         OGRLayer::SetMetadataItem("DESCRIPTION", osForcedDescription);
     }
 
-    if( (pszDomain == NULL || EQUAL(pszDomain, "")) &&
+    if( (pszDomain == nullptr || EQUAL(pszDomain, "")) &&
         osForcedDescription.empty() )
     {
         const char* l_pszDescription = OGRLayer::GetMetadataItem("DESCRIPTION");
@@ -1818,13 +1845,13 @@ CPLErr OGRPGDumpLayer::SetMetadata(char** papszMD, const char* pszDomain)
 CPLErr OGRPGDumpLayer::SetMetadataItem(const char* pszName, const char* pszValue,
                                        const char* pszDomain)
 {
-    if( (pszDomain == NULL || EQUAL(pszDomain, "")) && pszName != NULL &&
+    if( (pszDomain == nullptr || EQUAL(pszDomain, "")) && pszName != nullptr &&
         EQUAL(pszName, "DESCRIPTION") && !osForcedDescription.empty() )
     {
         return CE_None;
     }
     OGRLayer::SetMetadataItem(pszName, pszValue, pszDomain);
-    if( (pszDomain == NULL || EQUAL(pszDomain, "")) && pszName != NULL &&
+    if( (pszDomain == nullptr || EQUAL(pszDomain, "")) && pszName != nullptr &&
         EQUAL(pszName, "DESCRIPTION") )
     {
         SetMetadata( GetMetadata() );
